import { NextResponse, NextRequest } from 'next/server'
import { validateToken } from '@/lib/jwt'
import { db } from "@/lib/db"
 
interface orCardFilterProps {
  sanitized_name:    string
  OR:         orSetFilterProps[]
} 

interface orSetFilterProps {
  set_code:          string
}

interface cardEntryAPIProps {
  amount: number,
  sanitized_name: string,
}

export async function PUT(req: NextRequest) {
  try {
    const token = req?.headers.get("authorization")?.split(" ")[1]
    const { id, name, cards, url, commander_name } = await req.json()

    if(token == undefined) {
      return NextResponse.json({"message": "You did not provide a token."},{
        status: 401,
      });
    }
    
    if(!validateToken(token)) {
      return NextResponse.json({"message": "Your token is not valid."},{
        status: 401,
      });
    }

    if( id == undefined ) {
      return NextResponse.json({"message": "Wrong data in the request."},{
        status: 401,
      });
    }

    const deck_to_update = await db.deck.findFirst({
      where: {
        id: id
      },
      relationLoadStrategy: "join",
      include: {
        bset: {
          include: {
            sets: true
          }
        }
      }
    })

    if(deck_to_update == undefined) {
      return NextResponse.json({"message": "No deck with this ID."},{
        status: 401,
      });
    }

    if (cards.length != 0 && commander_name != undefined) {
      // Delete previous cards from deck
      await db.cartes_dans_deck.deleteMany({
        where: {
          deck_id: id
        }
      })

      const bset = deck_to_update?.bset

      const set_codes: orSetFilterProps[] = []
      bset?.sets.forEach((set) => {
        set_codes.push({set_code: set.code})
      })

      const cardsFilter: orCardFilterProps[] = [{sanitized_name: commander_name, OR: set_codes}]
      cards.forEach((card:cardEntryAPIProps) => {
        cardsFilter.push({sanitized_name: card.sanitized_name, OR: set_codes})
      })

      let cardsData = await db.carte.findMany({
        where: {
          OR: set_codes
        }
      })

      // Sort cards to select non promo types first and fallback to promo cards if not found
      cardsData = cardsData.sort((a,b) => +a.is_promo - +b.is_promo)
      
      let allCardFound = true
      if(cardsData.findIndex(cardData => cardData.sanitized_name == commander_name) == -1){
        if(cardsData.findIndex(cardData => cardData.sanitized_name.includes(commander_name)) == -1){
          allCardFound = false
        }
      }
      cards.forEach((card: cardEntryAPIProps) => {
        if(cardsData.findIndex(cardData => cardData.sanitized_name == card.sanitized_name) == -1){
          if(cardsData.findIndex(cardData => cardData.sanitized_name.includes(card.sanitized_name)) == -1 ){
            allCardFound = false
          }
        }
      })
      
      if(!allCardFound) {
        return NextResponse.json({"message": "Some cards were not found..."},{
          status: 401,
        });
      }

      const commander_card = cardsData.findIndex(cardData => cardData.sanitized_name == commander_name) == -1 ? cardsData[cardsData.findIndex(cardData => cardData.sanitized_name.includes(commander_name))] : cardsData[cardsData.findIndex(cardData => cardData.sanitized_name == commander_name)]


      cards.forEach(async (card: cardEntryAPIProps) => {
        const cardData_id = cardsData.findIndex(cardData => cardData.sanitized_name == card.sanitized_name) == -1 ? cardsData[cardsData.findIndex(cardData => cardData.sanitized_name.includes(card.sanitized_name))].id : cardsData[cardsData.findIndex(cardData => cardData.sanitized_name == card.sanitized_name)].id
        console.log(card.sanitized_name)
        await db.cartes_dans_deck.create({
          data: {
            amount: card.amount,
            carte: {
              connect: {
                id: cardData_id
              }
            },
            deck: {
              connect: {
                id: id
              }
            }
          }
        })
      })
      const deck = await db.deck.update({
        where: {
          id: id,
        },
        data: {
          name,
          url,
          color_identity: commander_card.color_identity,
          commander: {
            connect: {
              id: commander_card.id
            }
          },
        },
        relationLoadStrategy: "join",
        include: {
          cartes: {
            include: {
              carte: true
            }
          },
          commander: true
        }
      })

      return NextResponse.json({"data": deck, "message": "Deck created !"},{
        status: 200,
      })
    } else {
      const deck = await db.deck.update({
        where: {
          id: id,
        },
        data: {
          name,
          url,
        },
        relationLoadStrategy: "join",
        include: {
          cartes: {
            include: {
              carte: true
            }
          },
          commander: true
        }
      })

      return NextResponse.json({"data": deck, "message": "Deck created !"},{
        status: 200,
      })
    }
  } catch (error) {
    console.log(error)
    return NextResponse.json(
      { error: "Failed, check console" },
      {
        status: 500,
      }
    );
  }
}
